@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_90:addr.h,v 1.10 1996/02/28 14:26:04 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_91:addr.h,v 1.11 1996/12/11 21:04:05 woods Exp"
  */
 
 /*
@@ -138,13 +138,6 @@ struct transport_hints {
 #define ERR_CONFERR	0x00100000L	/* configuration error encountered */
 #define ERR_DONTLOG	0x00200000L	/* don't log this error */
 
-/*
- * user and group ids are preset to BOGUS_USER and BOGUS_GROUP in order
- * to prevent them from being mistaken for root and wheel
- */
-#define BOGUS_USER (-1)			/* not a valid user id */
-#define BOGUS_GROUP (-1)		/* not a valid group id */
-
 /* return values from parse_address */
 #define PARSE_ERROR	0		/* error in parsing */
 #define RFC_ROUTE	1		/* route part of a route-addr */
