@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_90:config.h,v 1.41 1996/11/26 20:57:03 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_91:config.h,v 1.42 1996/12/11 21:05:38 woods Exp"
  */
 
 /*
@@ -812,14 +812,26 @@ ${if def:input_addr\
 # define FNLOCK_MODE	0666		/* everybody can read and write */
 #endif
 
+/*
+ * user and group ids are preset to BOGUS_USER and BOGUS_GROUP in order
+ * to prevent them from being mistaken for root and wheel
+ */
+#ifndef BOGUS_USER
+# define BOGUS_USER	(-1)		/* not a valid user id */
+#endif
+#ifndef BOGUS_GROUP
+# define BOGUS_GROUP	(-1)		/* not a valid group id */
+#endif
+
 /*
  * Define a nobody user ID and group ID for defining default access
  * permissions.  This is often used by directors and routers and
  * transports that call external programs where no other uid/gid has
  * been given.
  *
- * if NOBODY_UID and NOBODY_GID are < 0, but SMAIL_NOBODY is non-NULL,
- * then getpwnam(3) is called to obtain the uid and gid.
+ * If NOBODY_UID (NOBODY_GID) are set to BOGUS_USER (BOGUS_GROUP), but
+ * SMAIL_NOBODY is non-NULL, then getpwnam(3) is called to obtain the value for
+ * nobody_uid (nobody_gid).
  *
  * NOTE: This can be set with the config file value "nobody"
  *
@@ -827,11 +839,11 @@ ${if def:input_addr\
  * #define NOBODY_GID	90		(* gid "guest" on namei *)
  */
 #ifndef	NOBODY_UID
-# define NOBODY_UID	(-1)		/* compute the nobody_uid */
+# define NOBODY_UID	BOGUS_USER	/* compute the nobody_uid */
 #endif
 
 #ifndef	NOBODY_GID
-# define NOBODY_GID	(-1)		/* compute the nobody_gid */
+# define NOBODY_GID	BOGUS_GROUP	/* compute the nobody_gid */
 #endif
 
 #ifndef	SMAIL_NOBODY
