@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_90:expand.c,v 1.19 1996/03/09 22:26:15 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_91:expand.c,v 1.20 1996/12/11 21:07:12 woods Exp"
  */
 
 /*
@@ -1521,7 +1521,7 @@ bad_subst(var, len)
     saved = xmalloc(len + 1);
     strncpy(saved, var, (size_t) len);
     saved[len] = '\0';
-    DEBUG1(DBG_DRIVER_MID, "expand_string: expansion failed for ${%s\n", saved);
+    DEBUG1(DBG_DRIVER_MID, "expand_string: expansion failed for ${%s}\n", saved);
     xfree(saved);
 }
 #endif
