@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_90:modes.c,v 1.56 1996/11/26 20:59:33 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_91:modes.c,v 1.57 1996/12/11 21:08:38 woods Exp"
  */
 
 /*
@@ -47,6 +47,7 @@
 # include <sys/ioctl.h>
 #endif
 #include <fcntl.h>
+#include "config.h"
 #include "smail.h"
 #include "alloc.h"
 #include "dys.h"
@@ -245,6 +246,8 @@ build_host_strings()
  *
  * if `nobody_uid' and `nobody_gid' are defined, use them, otherwise
  * use the login name in `nobody' to determine nobody_uid/gid.
+ *
+ * XXX assumes nobody's gid == group "nobody"
  */
 void
 compute_nobody()
@@ -265,7 +268,7 @@ compute_nobody()
 
 	pw = getpwbyname(nobody);
 	if (pw == NULL) {
-	    nobody_uid = UID_NOBODY;
+	    nobody_uid = UID_NOBODY;	/* XXX may still be == BOGUS_USER */
 	    nobody_gid = GID_NOBODY;
 	} else {
 	    nobody_uid = pw->pw_uid;
@@ -1461,13 +1464,14 @@ start_daemon()
 	    (void) fclose(stdin);
 	}
     }
+    (void) chdir("/");
 
     if (queue_interval > 0) {
 	daemon_pid = getpid();
 	write_log(LOG_SYS, "pid %d: smail daemon started", daemon_pid);
     }
 
-    /* toss the real uid under which smail was executed */
+    /* grab the real uid under which smail was executed */
     real_uid = getuid();
 
     return 0;
