@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_90:sysdep.c,v 1.51 1996/11/10 17:26:19 woods Exp
+#ident	"@(#)smail/src:RELEASE-3_2_0_91:sysdep.c,v 1.54 1997/01/15 02:42:56 woods Exp
  */
 
 /*
@@ -150,10 +150,16 @@
  * XXX /usr/5lib/libc.a{setuid.o} to ensure maximum bug compatability.
  * XXX So far as I can tell from disassembling /usr/lib/libc.a{setuid.o},
  * XXX the setuid() macro is a faithful re-implementation of that routine:
+ *
+ * XXX We do need to be sure we pretend to fail if we're trying to setuid to
+ * XXX -1 (the common value for BOGUS_USER), since setre?id() won't fail in
+ * XXX this case and we'd still be running as root when we thought we'd
+ * XXX changed to nobody_uid or some such less privileged user or group.
+ * XXX This will prevent use of uid 65535, but will allow -2 (65534).
  */
 #if defined(sun) && defined(UNIX_SUN_OS_4_1) && defined(UNIX_SYS5)
-# define setuid(n)	setreuid((-1), (n))
-# define setgid(n)	setregid((-1), (n))
+# define setuid(n)	((n) == (uid_t) -1 ? (errno = EINVAL, -1) : setreuid((-1), (n)))
+# define setgid(n)	((n) == (gid_t) -1 ? (errno = EINVAL, -1) : setregid((-1), (n)))
 #endif
 
 #if defined(FORCE_DIRSIZ_VALUE)
@@ -505,7 +511,7 @@ fopen_as_user(fn, type, uid, gid, mode)
     int fs;				/* temp status variable */
     struct stat stb_root, stb_user;	/* stat of file in root/user contexts */
 
-    DEBUG5(DBG_DRIVER_HI, "fopen_as_user(%s, %s, %d, %d, %d) called\n",
+    DEBUG5(DBG_DRIVER_HI, "fopen_as_user(%s, %s, %d, %d, 0%o) called\n",
 	   fn, type, uid, gid, mode);
     /* reject non-root-based paths out of hand, though we shouldn't get any */
     if (!fn || fn[0] != '/') {
@@ -524,34 +530,36 @@ fopen_as_user(fn, type, uid, gid, mode)
      */
     if (geteuid() == 0 && uid != 0 && gid != 0) {
         /*
-	 * If writing, check whether the file already exists as we need it to
-	 * do the access check properly.
+	 * If writing (incl. appending), check whether the file already exists
+	 * as we need it to do the access check properly.
 	 */
 	if (((type[0] == 'w') || (type[0] == 'a')) && (access(fn, F_OK) == EOF)) {
 	    /*
-	     * OK, we need to fork here to create the file....  If access()
-	     * failed we don't care why, as open() will discover the reason
-	     * soon enough.
+	     * OK, we need to create the file as the user, so we fork()....  If
+	     * access() failed we don't care why, as open() will discover the
+	     * reason soon enough.
 	     */
 	    switch ((pid = fork())) {
 	    case EOF:
-		DEBUG1(DBG_DRIVER_LO, "fopen_as_user(): fork(): %s.\n", strerror(errno));
+		DEBUG1(DBG_DRIVER_LO, "fopen_as_user(): create child fork(): %s.\n", strerror(errno));
 		return NULL;
 		/* NOTREACHED */
 
 	    case 0:
 		if (setgid(gid) != 0) {
-		    DEBUG2(DBG_DRIVER_LO, "fopen_as_user(): setgid(%d): %s.\n", gid, strerror(errno));
+		    DEBUG3(DBG_DRIVER_LO, "fopen_as_user(): create child setgid(%d): %s (uid was %d).\n",
+			   gid, strerror(errno), uid);
 		    _exit(errno + 1);
 		    /* NOTREACHED */
 		}
 		if (setuid(uid) != 0) {
-		    DEBUG2(DBG_DRIVER_LO, "fopen_as_user(): setuid(%d): %s.\n", uid, strerror(errno));
+		    DEBUG2(DBG_DRIVER_LO, "fopen_as_user(): create child setuid(%d): %s.\n",
+			   uid, strerror(errno));
 		    _exit(errno + 1);
 		    /* NOTREACHED */
 		}
 		if ((cfd = open(fn, O_WRONLY | O_CREAT, mode)) < 0) {
-		    DEBUG2(DBG_DRIVER_LO, "fopen_as_user(): test open(%s): %s.\n", fn, strerror(errno));
+		    DEBUG2(DBG_DRIVER_LO, "fopen_as_user(): create open(%s): %s.\n", fn, strerror(errno));
 		    _exit(errno + 1);
 		    /* NOTREACHED */
 		}
@@ -563,20 +571,20 @@ fopen_as_user(fn, type, uid, gid, mode)
 		/* wait for the child process to die */
 		while ((i = wait((STATUS_TYPE *) &status)) != pid) {
 		    if (i < 0) {
-			DEBUG2(DBG_DRIVER_MID, "fopen_as_user(): test child wait() for %d: %s.\n",
+			DEBUG2(DBG_DRIVER_MID, "fopen_as_user(): create child wait() for %d: %s.\n",
 			       pid, strerror(errno));
 			return NULL;
 		    }
 		}
 		if (WIFEXITED(status) && WEXITSTATUS(status)) {
 		    DEBUG1(WEXITSTATUS(status) - 1 ? DBG_DRIVER_LO : DBG_DRIVER_HI,
-			   "fopen_as_user(): test child exited with %d.\n",
+			   "fopen_as_user(): create child exited with %d.\n",
 			   WEXITSTATUS(status) - 1);
 		    errno = WEXITSTATUS(status) - 1;
 		    return NULL;
 		}
 		if (WIFSIGNALED(status) || WIFSTOPPED(status)) {
-		    DEBUG2(DBG_DRIVER_LO, "fopen_as_user(): test child[%d] signalled %d.\n",
+		    DEBUG2(DBG_DRIVER_LO, "fopen_as_user(): create child[%d] signalled %d.\n",
 			   pid, WTERMSIG(status));	/* XXX what if stopped???? */
 		    errno = EINTR;
 		    return NULL;
@@ -603,7 +611,7 @@ fopen_as_user(fn, type, uid, gid, mode)
 	if (fstat(fd, &stb_root) != 0) {
 	    int oerrno = errno;
 
-	    DEBUG2(DBG_DRIVER_LO, "fopen_as_user(): fstat(%s): %s.\n", fn, strerror(errno));
+	    DEBUG2(DBG_DRIVER_LO, "fopen_as_user(): root fstat(%s): %s.\n", fn, strerror(errno));
 	    close(fd);
 	    errno = oerrno;
 	    return NULL;
@@ -613,7 +621,7 @@ fopen_as_user(fn, type, uid, gid, mode)
 	    {
 		int oerrno = errno;
 
-		DEBUG1(DBG_DRIVER_LO, "fopen_as_user(): fork(): %s.\n", strerror(errno));
+		DEBUG1(DBG_DRIVER_LO, "fopen_as_user(): test child fork(): %s.\n", strerror(errno));
 		close(fd);
 		errno = oerrno;
 	    }
@@ -622,12 +630,14 @@ fopen_as_user(fn, type, uid, gid, mode)
 
 	case 0:
 	    if (setgid(gid) != 0) {
-		DEBUG2(DBG_DRIVER_LO, "fopen_as_user(): setgid(%d): %s.\n", gid, strerror(errno));
+		DEBUG3(DBG_DRIVER_LO, "fopen_as_user(): test child setgid(%d): %s (uid was %d).\n",
+		       gid, strerror(errno), uid);
 		_exit(errno + 1);
 		/* NOTREACHED */
 	    }
 	    if (setuid(uid) != 0) {
-		DEBUG2(DBG_DRIVER_LO, "fopen_as_user(): setuid(%d): %s.\n", uid, strerror(errno));
+		DEBUG2(DBG_DRIVER_LO, "fopen_as_user(): test childsetuid(%d): %s.\n",
+		       uid, strerror(errno));
 		_exit(errno + 1);
 		/* NOTREACHED */
 	    }
@@ -643,14 +653,16 @@ fopen_as_user(fn, type, uid, gid, mode)
 		cfd = open(fn, O_RDONLY, mode);
 	    }
 	    if (cfd < 0) {
-		DEBUG2(DBG_DRIVER_LO, "fopen_as_user(): child open(%s): %s.\n", fn, strerror(errno));
+		DEBUG2(DBG_DRIVER_LO, "fopen_as_user(): test child open(%s): %s.\n",
+		       fn, strerror(errno));
 		_exit(errno + 1);
 		/* NOTREACHED */
 	    }
 	    if (fstat(cfd, &stb_user) < 0) {
 		int oerrno = errno;
 
-		DEBUG2(DBG_DRIVER_LO, "fopen_as_user(): child fstat(%s): %s.\n", fn, strerror(errno));
+		DEBUG2(DBG_DRIVER_LO, "fopen_as_user(): test child fstat(%s): %s.\n",
+		       fn, strerror(errno));
 		close(cfd);
 		_exit(oerrno + 1);
 	    }
@@ -668,20 +680,20 @@ fopen_as_user(fn, type, uid, gid, mode)
 	    /* wait for the child process to die */
 	    while ((i = wait((STATUS_TYPE *) &status)) != pid) {
 		if (i < 0) {
-		    DEBUG2(DBG_DRIVER_MID, "fopen_as_user(): child wait() for %d: %s.\n",
+		    DEBUG2(DBG_DRIVER_MID, "fopen_as_user(): test child wait() for %d: %s.\n",
 			   pid, strerror(errno));
 		    return NULL;
 		}
 	    }
 	    if (WIFEXITED(status) && WEXITSTATUS(status)) {
 		DEBUG1(WEXITSTATUS(status) - 1 ? DBG_DRIVER_LO : DBG_DRIVER_HI,
-		       "fopen_as_user(): child exited with %d.\n",
+		       "fopen_as_user(): test child exited with %d.\n",
 		       WEXITSTATUS(status) - 1);
 		errno = WEXITSTATUS(status) - 1;
 		return NULL;
 	    }
 	    if (WIFSIGNALED(status) || WIFSTOPPED(status)) {
-		DEBUG2(DBG_DRIVER_LO, "fopen_as_user(): child[%d] signalled %d.\n",
+		DEBUG2(DBG_DRIVER_LO, "fopen_as_user(): test child[%d] signalled %d.\n",
 		       pid, WTERMSIG(status));	/* XXX what if stopped? */
 		errno = EINTR;
 		return NULL;
@@ -694,7 +706,7 @@ fopen_as_user(fn, type, uid, gid, mode)
 	return NULL;
     }
 
-    DEBUG5(DBG_DRIVER_HI, "fopen_as_user(%s, %s, %d, %d, %d) returning\n",
+    DEBUG5(DBG_DRIVER_HI, "fopen_as_user(%s, %s, %d, %d, 0%o) returning\n",
 	   fn, type, uid, gid, mode);
     return fp;
 }
@@ -1468,6 +1480,7 @@ open_child(argv, envp, in, out, errfd, flags, uid, gid)
 	}
 
 	/* change uid and gid, if we can, don't bother to check */
+	/* XXX This makes me very uncomfortable.  Why don't we check? */
 	(void) setgid(gid);
 	(void) setuid(uid);
 
